package com.dashitech.migration.core;

import com.dashitech.migration.util.JdbcRow;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 数据迁移处理
 */
public class DataMigrationter {
    public static Log log = LogFactory.getLog(DataMigrationter.class);

    /**
     * 根据loadCache的数据，区分是新增，还是修改
     */
    public static RowsWrap getRowsWrap(FieldMapper dataMapper,FieldMapper cacheMapper) {
        RowsWrap rw = new RowsWrap();
        List<JdbcRow> rows = dataMapper.getRows();
        Map<String, Long> targetCache = cacheMapper.getTargetCache();
        if(rows.size() > 0) {
            for (JdbcRow jdbcRow : rows) {
                Map<String, Object> row = jdbcRow.getRow();

                //拼接联合主键，判断是更新，还是新增
                StringBuffer key = new StringBuffer();
                for (DataMapper fieldMapper : cacheMapper.getFieldList()) {
                    Object value = row.get(fieldMapper.getTargetName());
                    //可能构建联合主键时，需要替换成新数据
                    value = getOprationReplaceValue(dataMapper, fieldMapper, value);
                    key.append(value).append("_");
                }
                key.deleteCharAt(key.length() - 1);

                //如果不存在是新增，如果存在，是修改
                Object value = targetCache.get(key.toString());
                if(value == null) {
                    rw.addInsertRow(jdbcRow);
                }else {
                    jdbcRow.setWhere("id", Long.valueOf(value.toString()));
                    rw.addUpdateRow(jdbcRow);
                }
            }
        }
        return rw;
    }

    /**
     * 开始匹配数据,并缓存到mapper的idCache里面
     */
    public static void loadCache(List<Map<String, Object>> targetList, List<Map<String, Object>> sourceList, FieldMapper mapper) {
        mapperData(targetList, sourceList, mapper);
        trans2IdCache(mapper);
    }

    /**
     * 待匹配字段，根据fieldList组成联合key，缓存成联合主键key:id
     */
    public static List<FieldMapper> mapperData(List<Map<String, Object>> targetList, List<Map<String, Object>> sourceList, FieldMapper mapper) {
        List<FieldMapper> result = new ArrayList<>();
        if (targetList != null && targetList.size() > 0 && sourceList != null && sourceList.size() > 0 && mapper != null && mapper.getFieldList().size() > 0) {

            //源数据
            for (Map<String, Object> map : sourceList) {
                setSourceKey(map, mapper);
            }

            //目标数据
            for (Map<String, Object> map : targetList) {
                setTargetKey(map, mapper);
            }

        }
        return result;
    }

    /**
     * key:源数据id,value:目标数据id
     */
    public static void trans2IdCache(FieldMapper mapper) {
        Map<Long, Long> cache = new HashMap<>();
        Map<String, Long> sourceCache = mapper.getSourceCache();
        for (Map.Entry<String, Long> entry : mapper.getTargetCache().entrySet()) {
            Long sourceId = sourceCache.get(entry.getKey());
            Long targetId = entry.getValue();
            if (sourceId != null && targetId != null) {
                cache.put(sourceId, targetId);
            } else if (sourceId != null) {
                cache.put(sourceId, null);
            }
        }
        mapper.setIdCache(cache);
    }


    /**
     * 设置源数据，联合主键
     */
    public static void setSourceKey(Map<String, Object> map, FieldMapper mapper) {
        StringBuffer source = new StringBuffer();
        Object id = map.get("id");
        for (DataMapper dataMapper : mapper.getFieldList()) {
            Object value = map.get(dataMapper.getSourceName());
            //可能构建联合主键时，需要替换成新数据
            value = getOprationReplaceValue(mapper, dataMapper, value);
            source.append(value).append("_");
        }
        source.deleteCharAt(source.length() - 1);
        mapper.getSourceCache().put(source.toString(), id == null ? null : Long.valueOf(id.toString()));
    }

    /**
     * 设置目标数据，联合主键
     */
    public static void setTargetKey(Map<String, Object> map, FieldMapper mapper) {
        StringBuffer target = new StringBuffer();
        Object id = map.get("id");
        for (DataMapper dataMapper : mapper.getFieldList()) {
            Object value = map.get(dataMapper.getTargetName());
            target.append(value).append("_");
        }
        target.deleteCharAt(target.length() - 1);
        mapper.getTargetCache().put(target.toString(), id == null ? null : Long.valueOf(id.toString()));
    }

    /**
     * 构建联合主键时，替换成新数据
     */
    private static Object getOprationReplaceValue(FieldMapper mapper, DataMapper dataMapper, Object value) {
        for (DataMapper opMapper : mapper.getRowsOperationList()) {
            //如果是要替换的，使用id缓存替换。
            if (opMapper.isReplace() && opMapper.getSourceName().equals(dataMapper.getSourceName()) && value != null) {
                Long valueId = opMapper.getIdCache().get(value);
                return valueId;
            }
        }
        return value;
    }

}
