package com.dashitech.migration.task.incident;

import com.dashitech.migration.BaseTask;
import com.dashitech.migration.SqlTask;
import com.dashitech.migration.core.DataMigrationter;
import com.dashitech.migration.core.FieldMapper;
import com.dashitech.migration.util.JdbcUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * Created by CX on 2024/10/10.
 */
@Component
public class IncidentLogTask extends BaseTask {

    @Autowired
    SqlTask userTask;
    @Autowired
    SqlTask groupTask;
    @Autowired
    SqlTask hospitalTask;
    @Autowired
    SqlTask dictionaryTask;

    public static Log log = LogFactory.getLog(IncidentLogTask.class);

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void start() {
        log.info(" *** IncidentLogTask start 开始 *** ");

        StringBuffer sb = new StringBuffer();
        sb.append(" select * from itsm_incident_log ");

        List list = jdbcTemplateMdv2.queryForList(sb.toString());

        FieldMapper mapper = new FieldMapper();
        Map<Long, Long> userCache = userTask.loadCache().getIdCache();
        mapper.addOperationUpdate("hos_id", "branch_id");
        mapper.addOperationReplace("log_type", dictionaryTask.loadCache().getIdCache());
        mapper.addOperationReplace("appointor_id", userCache);
        mapper.addOperationReplace("worker_id", userCache);
        mapper.addOperationReplace("group_id", groupTask.loadCache().getIdCache());
        mapper.addOperationReplace("duty_id", hospitalTask.loadCache().getIdCache());
        mapper.trans2JdbcRow(list);

        JdbcUtil jdbcUtil = new JdbcUtil(jdbcTemplateHsms);
        jdbcUtil.batchPageInsert("itsm_itsm_incident_log", mapper.getRows());

        //clear释放内存
        mapper.clear();
    }

    @Override
    public FieldMapper loadCache() {
        log.info(" *** IncidentLogTask loadCache 开始 *** ");


        return null;
    }
}
