package com.dashitech.migration.task.uc;

import com.dashitech.migration.BaseTask;
import com.dashitech.migration.SqlTask;
import com.dashitech.migration.core.DataMigrationter;
import com.dashitech.migration.core.FieldMapper;
import com.dashitech.migration.util.JdbcUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * Created by CX on 2024/10/10.
 */
@Component
public class CompanyTask extends BaseTask {

    public static Log log = LogFactory.getLog(CompanyTask.class);

    @Autowired
    SqlTask hospitalTask;
    @Autowired
    SqlTask dictionaryTask;



    @Override
    @Transactional(rollbackFor = Exception.class)
    public void start() {
        log.info(" *** 三方公司CompanyTask转换开始 *** ");

        StringBuffer sb = new StringBuffer();
        sb.append(" select version,name,contacts,mphone,service_type_ids as 'company_type_ids', ");
        sb.append(" delete_flag,add_time,branch_id as 'hos_id' ");
        sb.append(" from itsm_uc_company ");

        List list = jdbcTemplateMdv2.queryForList(sb.toString());

        FieldMapper mapper = new FieldMapper();
        mapper.addOperationAdd("busi_type", getDicId(jdbcTemplateHsms,"company_busi_type","1")); //公司业务类型-医废
        Map<Long, Long> hosIdCache = hospitalTask.loadCache().getIdCache();
        mapper.addOperationReplace("hos_id", hosIdCache);
        mapper.addOperationReplace("company_type_ids", dictionaryTask.loadCache().getIdCache());
        mapper.trans2JdbcRow(list);

        JdbcUtil jdbcUtil = new JdbcUtil(jdbcTemplateHsms);
        jdbcUtil.batchPageInsert("itsm_uc_company", mapper.getRows());

        //clear释放内存
        mapper.clear();
    }

    @Override
    public FieldMapper loadCache() {
        log.info(" *** 加载三方公司CompanyTask缓存 *** ");

        String sql = " select id,name,delete_flag from itsm_uc_company ";
        List<Map<String, Object>> list = jdbcTemplateHsms.queryForList(sql);
        sql = " select id,name,delete_flag from itsm_uc_company ";
        List<Map<String, Object>> list2 = jdbcTemplateMdv2.queryForList(sql);

        FieldMapper mapper = new FieldMapper();
        mapper.addFieldMapper("name", "name");
        mapper.addFieldMapper("delete_flag", "delete_flag");

        DataMigrationter.loadCache(list, list2, mapper);

        return mapper;
    }
}
