package com.dashitech.migration.task.uc;

import com.dashitech.migration.BaseTask;
import com.dashitech.migration.core.DataMigrationter;
import com.dashitech.migration.core.FieldMapper;
import com.dashitech.migration.core.RowsWrap;
import com.dashitech.migration.util.JdbcUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * @author song
 * @date 2024年10月12日
 */
@Component
public class DepartmentTask extends BaseTask {

    public static Log log = LogFactory.getLog(DepartmentTask.class);

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void start() {
        log.info(" *** DepartmentTask start 开始 *** ");
        StringBuffer sql = new StringBuffer("SELECT dept,CODE,parentId,email,delete_flag,address as _address,place_id as floor_id,area_id as building_id,pinyin,");
        sql.append("branch_id as _hospital_id,dept_alias,input_code,phone as many_phone,qrcode,1 as flag,IF(phone is not null,SUBSTRING_INDEX(phone,',',1),null) as _phone ");
        sql.append(" FROM itsm_uc_dept where id in (select distinct b.id from itsm_incident a join itsm_uc_dept b on a.dept = b.id where a.dept > 1204)");
        List<Map<String, Object>> deptList = jdbcTemplateMdv2.queryForList(sql.toString());

        FieldMapper mapper = new FieldMapper();
        mapper.addOperationAdd("_type", getDicId(jdbcTemplateHsms, "dept_type", "other"));
        mapper.trans2JdbcRow(deptList);

        FieldMapper deptMapper = this.loadCache();
        RowsWrap rw = DataMigrationter.getRowsWrap(mapper,deptMapper);
        JdbcUtil jdbcUtil = new JdbcUtil(jdbcTemplateHsms);
        //科室只做更新 已有的科室不做处理
        jdbcUtil.batchPageInsert("itsm_uc_dept", rw.getInsertRows());

        mapper.clear();
        deptMapper.clear();
    }

    @Override
    public FieldMapper loadCache() {
        log.info(" *** DepartmentTask load开始 *** ");
        StringBuffer sql = new StringBuffer("select id,dept from itsm_uc_dept");
        List<Map<String, Object>> hsmsDeptList = jdbcTemplateHsms.queryForList(sql.toString());
        List<Map<String, Object>> itsmDeptList = jdbcTemplateMdv2.queryForList(sql.toString());

        FieldMapper mapper = new FieldMapper();
        mapper.addFieldMapper("dept", "dept");

        DataMigrationter.loadCache(hsmsDeptList, itsmDeptList, mapper);

        return mapper;
    }
}
