package com.dashitech.migration.task.uc;

import com.dashitech.migration.BaseTask;
import com.dashitech.migration.SqlTask;
import com.dashitech.migration.core.DataMigrationter;
import com.dashitech.migration.core.FieldMapper;
import com.dashitech.migration.util.JdbcUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * Created by CX on 2024/10/10.
 */
@Component
public class GroupTask extends BaseTask {

    public static Log log = LogFactory.getLog(GroupTask.class);

    @Autowired
    SqlTask hospitalTask;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void start() {
        log.info(" *** 工作组group转换开始 *** ");

        StringBuffer sql = new StringBuffer();
        if ("xiangyang".equals(appName)) {
            sql.append("select 0 as version,a.groupname,a.delete_flag,a.duty_id ,now() as operator_time,1 as schedule_class_id");
            sql.append(" from itsm_uc_groups a join itsm_uc_duty_dept b on a.duty_id = b.id ");
            sql.append("where b.dept not like 'Z%' and dept not like '%保卫%' and b.id not in (20,21)");
        } else if ("nanjing".equals(appName)) {
            sql.append(" select * from itsm_uc_groups ");
        }


        List list = jdbcTemplateMdv2.queryForList(sql.toString());

        //测试
        FieldMapper mapper = new FieldMapper();
        mapper.addOperationAdd("type", 3);
        Map<Long, Long> hosIdCache = hospitalTask.loadCache().getIdCache();
        if ("xiangyang".equals(appName)) {
            mapper.addOperationUpdate("_hospital_id", "duty_id");
            mapper.addOperationReplace("duty_id", hosIdCache);
        } else if ("nanjing".equals(appName)) {
            mapper.addOperationRemove("id").addOperationRemove("duty_id");
            mapper.addOperationUpdate("_hospital_id", "branch_id");
            mapper.addOperationReplace("branch_id", hosIdCache);
        }
        mapper.trans2JdbcRow(list);

        JdbcUtil jdbcUtil = new JdbcUtil(jdbcTemplateHsms);
        jdbcUtil.batchPageInsert("itsm_uc_groups", mapper.getRows());

        //clear释放内存
        mapper.clear();
    }

    @Override
    public FieldMapper loadCache() {
        log.info(" *** 加载工作组group缓存 *** ");

        String sql = " select id,groupname,delete_flag from itsm_uc_groups ";
        List<Map<String, Object>> list = jdbcTemplateHsms.queryForList(sql);
        sql = " select id,groupname,delete_flag from itsm_uc_groups ";
        List<Map<String, Object>> list2 = jdbcTemplateMdv2.queryForList(sql);

        FieldMapper mapper = new FieldMapper();
        mapper.addFieldMapper("groupname", "groupname");
        mapper.addFieldMapper("delete_flag", "delete_flag");

        DataMigrationter.loadCache(list, list2, mapper);

        return mapper;
    }
}
