package com.dashitech.migration.task.uc;

import com.dashitech.migration.BaseTask;
import com.dashitech.migration.core.DataMigrationter;
import com.dashitech.migration.core.FieldMapper;
import com.dashitech.migration.util.JdbcUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by CX on 2024/10/10.
 */
@Component
public class HospitalTask extends BaseTask {

    public static Log log = LogFactory.getLog(HospitalTask.class);

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void start() {
        log.info(" *** HospitalTask start开始 *** ");
        if ("xiangyang".equals(appName)) {
            startXy();
        } else {
            startNj();
        }
    }

    public void startXy() {
        StringBuffer sql = new StringBuffer("SELECT 0 as version,branch_id as parentId,dept as _hos_name,IFNULL(code,py) as _hos_no,delete_flag,now() as operator_time ");
        sql.append(" FROM `itsm_uc_duty_dept` where dept not like 'Z%'  and dept not like '%保卫%' and id not in (20,21)");
        List<Map<String, Object>> hosList = jdbcTemplateMdv2.queryForList(sql.toString());
        FieldMapper mapper = new FieldMapper();
        mapper.addOperationAdd("type", getDicId(jdbcTemplateHsms,"hospital_type","6"));//院区类型 维修科室
        mapper.trans2JdbcRow(hosList);
        JdbcUtil jdbcUtil = new JdbcUtil(jdbcTemplateHsms);
        jdbcUtil.batchPageInsert("itsm_uc_hospital", mapper.getRows());

        //clear释放内存
        mapper.clear();
    }

    public void startNj() {
        StringBuffer sql = new StringBuffer("SELECT 0 as version,branch_id as parentId,dept as _hos_name,IFNULL(code,py) as _hos_no,delete_flag,now() as operator_time ");
        sql.append(" FROM `itsm_uc_duty_dept` ");
        List<Map<String, Object>> hosList = jdbcTemplateMdv2.queryForList(sql.toString());
        FieldMapper mapper = new FieldMapper();
        mapper.addOperationAdd("type", getDicId(jdbcTemplateHsms,"hospital_type","6"));//院区类型 维修科室
        mapper.trans2JdbcRow(hosList);
        JdbcUtil jdbcUtil = new JdbcUtil(jdbcTemplateHsms);
        jdbcUtil.batchPageInsert("itsm_uc_hospital", mapper.getRows());

        //clear释放内存
        mapper.clear();
    }

    @Override
    public FieldMapper loadCache() {
        log.info(" *** 加载院区Hospital缓存 *** ");

        FieldMapper mapper = null;
        if ("xiangyang".equals(appName)) {
            mapper = loadXiangyangCache();
        } else {
            mapper = loadNanjingCache();
        }

        return mapper;
    }

    public FieldMapper loadXiangyangCache() {
        StringBuffer sql = new StringBuffer("SELECT id,dept FROM `itsm_uc_duty_dept` where dept not like 'Z%'  and dept not like '%保卫%' and id not in (20,21)");
        List<Map<String, Object>> dutyList = jdbcTemplateMdv2.queryForList(sql.toString());
        sql = new StringBuffer("select id,_hos_name from itsm_uc_hospital where type = (select id from itsm_base_dictionary where _key = 'hospital_type' and _value = '6')");
        List<Map<String, Object>> hosList = jdbcTemplateHsms.queryForList(sql.toString());
        FieldMapper mapper = new FieldMapper();
        mapper.addFieldMapper("_hos_name", "dept");

        DataMigrationter.loadCache(hosList, dutyList, mapper);

        return mapper;
    }

    public FieldMapper loadNanjingCache() {
        StringBuffer sql = new StringBuffer("SELECT id,dept FROM `itsm_uc_duty_dept` ");
        List<Map<String, Object>> dutyList = jdbcTemplateMdv2.queryForList(sql.toString());
        sql = new StringBuffer("select id,_hos_name from itsm_uc_hospital ");
        List<Map<String, Object>> hosList = jdbcTemplateHsms.queryForList(sql.toString());
        FieldMapper mapper = new FieldMapper();
        mapper.addFieldMapper("_hos_name", "dept");

        DataMigrationter.loadCache(hosList, dutyList, mapper);

        return mapper;
    }
}
