package com.dashitech.migration.task.uc;

import com.dashitech.migration.BaseTask;
import com.dashitech.migration.SqlTask;
import com.dashitech.migration.core.DataMigrationter;
import com.dashitech.migration.core.FieldMapper;
import com.dashitech.migration.core.RowsWrap;
import com.dashitech.migration.util.JdbcUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * Created by CX on 2024/10/10.
 */
@Component
public class UserTask extends BaseTask {

    @Autowired
    SqlTask hospitalTask;

    @Autowired
    SqlTask departmentTask;

    public static Log log = LogFactory.getLog(UserTask.class);

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void start() {
        log.info(" *** 用户user转换开始 *** ");

        StringBuffer sb = new StringBuffer();
        sb.append(" select version,ACCOUNT,GENDER,NAME,PHONE,PWD,WEIXIN,DEPTID,delete_flag, ");
        sb.append(" pinyin,py as 'input_code',online,branch_id as 'hospital_id', ");
        sb.append(" duty_id as 'duty_dept',company as 'company_id',engineer ");
        sb.append(" from itsm_uc_user ");
        sb.append(" where ACCOUNT != 'dsadmin' ");

        List list = jdbcTemplateMdv2.queryForList(sb.toString());

        FieldMapper mapper = new FieldMapper();
        mapper.addOperationAdd("usertype", getDicId(jdbcTemplateHsms,"usertype","4")); //用户类型-默认三方人员
        mapper.addOperationReplace("hospital_id", hospitalTask.loadCache().getIdCache());
        mapper.addOperationReplace("duty_dept", hospitalTask.loadCache().getIdCache());
        if ("xiangyang".equals(appName)) {
            mapper.addOperationReplace("DEPTID", departmentTask.loadCache().getIdCache());
        }
        mapper.trans2JdbcRow(list);

        FieldMapper userMapper = loadCache();
        RowsWrap rw = DataMigrationter.getRowsWrap(mapper,userMapper);
        JdbcUtil jdbcUtil = new JdbcUtil(jdbcTemplateHsms);
        jdbcUtil.batchPageInsert("itsm_uc_user", rw.getInsertRows());
        jdbcUtil.batchPageUpdate("itsm_uc_user", rw.getUpdateRows());

        //clear释放内存
        mapper.clear();
        userMapper.clear();
    }

    @Override
    public FieldMapper loadCache() {
        log.info(" *** 加载用户user缓存 *** ");

        String sql = " select id,ACCOUNT,delete_flag from itsm_uc_user ";
        List<Map<String, Object>> list = jdbcTemplateHsms.queryForList(sql);
        sql = " select id,ACCOUNT,delete_flag from itsm_uc_user ";
        List<Map<String, Object>> list2 = jdbcTemplateMdv2.queryForList(sql);

        FieldMapper mapper = new FieldMapper();
        mapper.addFieldMapper("ACCOUNT", "ACCOUNT");
        mapper.addFieldMapper("delete_flag", "delete_flag");

        DataMigrationter.loadCache(list, list2, mapper);

        return mapper;
    }
}
